/**
 * @author Florin Mihut
 **/
/**
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

const ExtensionUtils = imports.misc.extensionUtils;
const Config = imports.misc.config;
const Lang = imports.lang;
const St = imports.gi.St;
const Main = imports.ui.main;
const Mainloop = imports.mainloop;
const Gtk = imports.gi.Gtk;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const Clutter = imports.gi.Clutter;
const Me = imports.misc.extensionUtils.getCurrentExtension();
const Settings = Me.imports.settings;
const Util = imports.misc.util;
const Providers = Me.imports.providers;

let _indicator;
let settings_signals = [];

const RouteIndicatorPosition = {
    CENTER: 0,
    RIGHT: 1,
    LEFT: 2
};

let XSIZE = 500;
const YSIZE = 180;
const YBORDER = 40;

const RouteIndicator = new Lang.Class({
    Name: 'RouteIndicator',
    Extends: PanelMenu.Button,

    _init: function() {
        // not sure what this is doing
        this.parent(0.25, "Route Indicator", false);

        this.flipOriginAndDestination = false;

        //todo: instantiate the route provider depending of the settings
        if (Settings.gsettings.get_string(Settings.COMMUTE_PROVIDER) === 'google') {
            this.routeProvider = new Providers.GoogleRouteProvider(this);
        } else {
            this.routeProvider = new Providers.BingRouteProvider(this);
        }

        // Panel menu item - the current class
        let menuAlignment = 0.25;
        if (Clutter.get_default_text_direction() == Clutter.TextDirection.RTL)
            menuAlignment = 1.0 - menuAlignment;
        this.parent(menuAlignment);

        // Label
        this.routeDurationInfo = new St.Label({
            y_align: Clutter.ActorAlign.CENTER,
            text: _(' Time')
        });

        this._iconActor = new St.Icon({
            icon_name: "security-high-symbolic",
            style_class: "system-status-icon"
        });

        // Putting the top panel items together
        let topBox = new St.BoxLayout();
        topBox.add_actor(this._iconActor);
        topBox.add_actor(this.routeDurationInfo);
        this.actor.add_actor(topBox);

        let dummyBox = new St.BoxLayout();
        this.actor.reparent(dummyBox);
        dummyBox.remove_actor(this.actor);
        dummyBox.destroy();

        let indicator_position;
        let indicator_position_as_string = Settings.gsettings.get_string(Settings.COMMUTE_INDICATOR_POSITION);
        if (indicator_position_as_string === 'left') {
            indicator_position = RouteIndicatorPosition.LEFT;
        } else if (indicator_position_as_string === 'right') {
            indicator_position = RouteIndicatorPosition.RIGHT;
        } else if (indicator_position_as_string === 'center') {
            indicator_position = RouteIndicatorPosition.CENTER;
        }

        let children = null;
        switch (indicator_position) {
            case RouteIndicatorPosition.LEFT:
                children = Main.panel._leftBox.get_children();
                Main.panel._leftBox.insert_child_at_index(this.actor,children.length);
                break;
            case RouteIndicatorPosition.CENTER:
                children = Main.panel._centerBox.get_children();
                Main.panel._centerBox.insert_child_at_index(this.actor,children.length);
                break;
            case RouteIndicatorPosition.RIGHT:
                children = Main.panel._rightBox.get_children();
                Main.panel._rightBox.insert_child_at_index(this.actor,0);
                break;
        }

        this.createMenu();
        // enter main loop for refreshing
        let refreshInterval = parseInt(Settings.gsettings.get_string(Settings.COMMUTE_REFRESH_INTERVAL));
        // in case of invalid value, consider updating the route every 30 minutes
        if ((refreshInterval === NaN) || (refreshInterval < 0)) refreshInterval = 30;
        if (refreshInterval > 0) {
            this._mainloop = Mainloop.timeout_add(refreshInterval*60000, Lang.bind(this, function() {
                // request new job states if auto-refresh is enabled
                if (false === Main.screenShield.locked) {
                    global.log("Commute: trigger autorefresh!");
                    this.routeProvider.reloadData();
                } else {
                    global.log("Commute: do not trigger autorefresh, user not active!");
                }
                // returning true is important for restarting the mainloop after timeout
                return true;
            }));
        };
        if (Settings.gsettings.get_boolean(Settings.COMMUTE_AUTOMATIC_SWITCH)) {
            this._mainloop = Mainloop.timeout_add(60000, Lang.bind(this, function() {
                // returning true is important for restarting the mainloop after timeout
                this._checkShouldFlipSAndD();
                return true;
            }));
        }
        // initial data
        this._checkShouldFlipSAndD();
        this.routeProvider.reloadData();
    },

    _checkShouldFlipSAndD: function() {
        // check if it is the time to switch the origin with destination
        let switchTime = new Date();
        let currentTime = new Date();
        let settingsSwitchTime = Settings.gsettings.get_string(Settings.COMMUTE_SWITCH_TIME);
        let splitTime = settingsSwitchTime.split(":");

        switchTime.setHours(splitTime[0]);
        switchTime.setMinutes(splitTime[1]);
        let toStringCurrentTime = currentTime.toTimeString();
        let toStringSwitchTime = switchTime.toTimeString();
        if ((toStringSwitchTime < toStringCurrentTime) && (false == this.flipOriginAndDestination)){
            this._onFlipOriginAndDestination();
        }
    },

    _onPreferencesActivate: function() {
        this.menu.actor.hide();
        Util.spawn(["gnome-shell-extension-prefs", "commute@florinmihut"]);
        return 0;
    },

   _onButtonHoverChanged: function(actor, event) {
        if (actor.hover) {
            actor.add_style_pseudo_class('hover');
            actor.set_style(this._button_background_style);
        } else {
            actor.remove_style_pseudo_class('hover');
            actor.set_style('background-color:;');
            if (actor != this._urlButton)
                actor.set_style(this._button_border_style);
        }
    },

    _onFlipOriginAndDestination: function() {
        this.flipOriginAndDestination = !this.flipOriginAndDestination;
        global.log(_("Commute: flipOriginAndDestination") + this.flipOriginAndDestination ? "true" : "false");
        // we should update the forecast data which if active might trigger a menu reload (or not)
        this.routeProvider.updateForecastData();
        this.reloadMenus();
    },

    createTopMenu: function() {
        if (Main.panel._menus === undefined)
            Main.panel.menuManager.addMenu(this.menu);
        else
            Main.panel._menus.addMenu(this.menu);

        let _itemFrom = new PopupMenu.PopupBaseMenuItem({
            reactive: false
        });
        let _itemTo = new PopupMenu.PopupBaseMenuItem({
            reactive: false
        });

        let labelText = _("From: ");
        if (this.flipOriginAndDestination) {
            labelText =  labelText + Settings.gsettings.get_string(Settings.COMMUTE_DESTINATION);
        } else {
            labelText =  labelText + Settings.gsettings.get_string(Settings.COMMUTE_ORIGIN);
        }
        this._fromLabel = new St.Label({
            y_align: Clutter.ActorAlign.CENTER,
            text: labelText
        });

        labelText = _("To: ");
        if (this.flipOriginAndDestination) {
            labelText =  labelText + Settings.gsettings.get_string(Settings.COMMUTE_ORIGIN);
        } else {
            labelText =  labelText + Settings.gsettings.get_string(Settings.COMMUTE_DESTINATION);
        }
        this._toLabel = new St.Label({
            y_align: Clutter.ActorAlign.CENTER,
            text: labelText
        });

        // try to detect the maximum size of the labels (from and to)
        let actualXSize = XSIZE; // we start with this size (minimum)
        let [fromSizeH,fromSizeV] = this._fromLabel.get_size();
        let [toSizeH,toSizeV] = this._toLabel.get_size();
        if (actualXSize < fromSizeH) actualXSize = fromSizeH;
        if (actualXSize < toSizeH) actualXSize = toSizeH;
        XSIZE = actualXSize; // make the current size to match the actual size computed above

        if (ExtensionUtils.versionCheck(['3.8'],Config.PACKAGE_VERSION)) {
            _itemTo.addActor(this._fromLabel);
            _itemFrom.addActor(this._toLabel);
        } else {
            _itemTo.actor.add_actor(this._fromLabel);
            _itemFrom.actor.add_actor(this._toLabel);
        }

        this.menu.addMenuItem(_itemTo);
        this.menu.addMenuItem(_itemFrom);

        this._separatorItem1 = new PopupMenu.PopupSeparatorMenuItem();
        this.menu.addMenuItem(this._separatorItem1);

        if (Settings.gsettings.get_boolean(Settings.COMMUTE_SWITCH_DISPLAYFORECAST)) {
            // here show the traffic forecast
            const scaleOnY = function(value, vmax, vmin) {
                return (YSIZE - YBORDER)/(vmax - vmin) * value + (YBORDER * vmax - YSIZE * vmin)/(vmax - vmin);
            }

            let forecastValues = this.routeProvider.getForecastValues();
            let vMin = this.routeProvider.getMinValue();
            let vMax = this.routeProvider.getMaxValue();

            const draw_stuff = function (canvas, cr, width, height) {
                cr.setLineWidth (1);

                // draw axes
                cr.setSourceRGBA(0.1, 0.75, 0.1, 0.5);
                for (let i=0; i < 25; i++) {
                    cr.moveTo(i * XSIZE / 24, 0);
                    cr.lineTo(i * XSIZE / 24, YSIZE);
                }
                cr.stroke ();

                // draw forecast values
                cr.moveTo(0, YSIZE + YBORDER / 2 - scaleOnY(forecastValues[0], vMax, vMin));
                cr.setSourceRGBA(1, 0, 0, 1);
                for (let i=0; i < forecastValues.length; i++) {
                    cr.lineTo(i * XSIZE / 24, YSIZE + YBORDER / 2 - scaleOnY(forecastValues[i], vMax, vMin));
                }
                if (forecastValues.length == 24) {
                    cr.lineTo(XSIZE, YSIZE + YBORDER / 2 - scaleOnY(forecastValues[0], vMax, vMin));
                }
                cr.stroke ();

                return true;
            };

            this._canvas = new Clutter.Canvas ();
            this._canvas.set_size (XSIZE, YSIZE);
            this._canvas.connect ("draw", draw_stuff);

            this._dummy = new Clutter.Actor ();
            this._dummy.set_content (this._canvas);
            this._dummy.set_size(XSIZE, YSIZE);

            var white = new Clutter.Color({red:255, blue:255, green:255, alpha:255});
            for (let h = 1; h < 8; h++) {
                let hours = new Clutter.Text({text:(3 * h).toString(), color:white});
                hours.set_anchor_point_from_gravity(Clutter.Gravity.CENTER);
                hours.x = 3 * h * XSIZE / 24;
                hours.y = YSIZE - YBORDER / 4;
                this._dummy.add_actor(hours);
                hours.show();
            }

            let vMaxLabel = new Clutter.Text({text:vMax.toString(), color:white});
            vMaxLabel.set_anchor_point_from_gravity(Clutter.Gravity.CENTER);
            vMaxLabel.x = 0;
            vMaxLabel.y = YSIZE + YBORDER / 2 - scaleOnY(vMax, vMax, vMin);
            this._dummy.add_actor(vMaxLabel);
            vMaxLabel.show();

            let vMinLabel = new Clutter.Text({text:vMin.toString(), color:white});
            vMinLabel.set_anchor_point_from_gravity(Clutter.Gravity.CENTER);
            vMinLabel.x = 0;
            vMinLabel.y = YSIZE + YBORDER / 2 - scaleOnY(vMin, vMax, vMin);
            this._dummy.add_actor(vMinLabel);
            vMinLabel.show();

            let vAvgLabel = new Clutter.Text({text:vMin.toString(), color:white});
            vAvgLabel.set_anchor_point_from_gravity(Clutter.Gravity.CENTER);
            vAvgLabel.x = 0;
            vAvgLabel.y = YSIZE + YBORDER / 2 - scaleOnY(vMin, vMax, vMin);
            this._dummy.add_actor(vAvgLabel);
            vAvgLabel.show();

            this._itemTraffic = new PopupMenu.PopupBaseMenuItem({
                reactive: false
            });

            if (ExtensionUtils.versionCheck(['3.8'],Config.PACKAGE_VERSION)) {
                this._itemTraffic.addActor(this._dummy);
            } else {
                this._itemTraffic.actor.add_actor(this._dummy);
            }
            this.menu.addMenuItem(this._itemTraffic);

            this._separatorItem2 = new PopupMenu.PopupSeparatorMenuItem();
            this.menu.addMenuItem(this._separatorItem2);
            this._canvas.invalidate();
        }
    },

    createBottomMenu: function() {
        // create the item holding the buttons at the base of the menu: reload, settings and the link to the routes
        let _itemButtons = new PopupMenu.PopupBaseMenuItem({
            reactive: false,
            style_class: 'commute-menu-button-container'
        });

        // add url and prefs
        this._buttonBox = new St.BoxLayout({
            x_expand: true,
            x_align: Clutter.ActorAlign.CENTER,
            style_class: 'commute-button-box'
        });

        this._reloadButton = this.createButton("view-refresh-symbolic", "Reload traffic info");
        this._reloadButton.connect('clicked', Lang.bind(this, function() {
            // trigger reload
            this.routeProvider.reloadData();
        }));
        this._buttonBox.add_actor(this._reloadButton);

        this._urlButton = this.createButton('', "Click to show the route in browser");
        this._urlButton.set_label("Show in browser");

        if (ExtensionUtils.versionCheck(['3.8'], Config.PACKAGE_VERSION)) {
            this._urlButton.connect('notify::hover', Lang.bind(this, this._onButtonHoverChanged));
            this._urlButton.style_class = 'popup-menu-item';
        }
        this._urlButton.style_class += ' commute-provider';

        this._urlButton.connect('clicked', Lang.bind(this, function() {
            // start default browser and navigate to url
            this.menu.actor.hide();
            let url = this.routeProvider.getBrowserURL();

            try {
                Gtk.show_uri(null, url, global.get_current_time());
            } catch (err) {
                let title = _("Can not open %s").format(url);
                Main.notifyError(title, err.message);
            }
        }));

        this._buttonBox.add_actor(this._urlButton);

        this._flipButton = this.createButton('object-flip-vertical-symbolic', _("Flip origin with destination"));
        this._flipButton.connect('clicked', Lang.bind(this, this._onFlipOriginAndDestination));
        this._buttonBox.add_actor(this._flipButton);

        this._prefsButton = this.createButton('preferences-system-symbolic', _("Commute Settings"));
        this._prefsButton.connect('clicked', Lang.bind(this, this._onPreferencesActivate));
        this._buttonBox.add_actor(this._prefsButton);

        if (ExtensionUtils.versionCheck(['3.8'], Config.PACKAGE_VERSION)) {
            this._buttonBox.add_style_class_name('commute-button-box-38');
            _itemButtons.addActor(this._buttonBox);
        } else {
            _itemButtons.actor.add_actor(this._buttonBox);
        }

        this.menu.addMenuItem(_itemButtons);
    },

    createMenu: function() {
        this.createTopMenu();
        this.createBottomMenu();
    },

    reloadMenus: function() {
        // need to destroy the previous menus
        let children = this.menu._getMenuItems();
        for (let i = 0; i < children.length; i++) {
             let item = children[i];
             item.destroy();
        }

        this.createTopMenu();
        this.routeProvider.createRoutesMenus();
        this.createBottomMenu();
    },

    createButton: function(iconName, accessibleName) {
        let button;
        if (ExtensionUtils.versionCheck(['3.8'],Config.PACKAGE_VERSION)) {
            button = new St.Button({
                reactive: true,
                can_focus: true,
                track_hover: true,
                accessible_name: accessibleName,
                style_class: 'commute-button'
            });
            button.child = new St.Icon({
                icon_name: iconName
            });
            button.connect('notify::hover', Lang.bind(this, this._onButtonHoverChanged));
        } else
            button = Main.panel.statusArea.aggregateMenu._system._createActionButton(iconName, accessibleName);
        return button;
    }
});

function init() {
    Clutter.init(null);
    Settings.init();
}

function enable() {
    settings_signals.push(Settings.gsettings.connect("changed::" + Settings.COMMUTE_INDICATOR_POSITION, function() {
        disable();
        enable();
    }));

    settings_signals.push(Settings.gsettings.connect("changed::" + Settings.COMMUTE_REFRESH_INTERVAL, function() {
        disable();
        enable();
    }));

    settings_signals.push(Settings.gsettings.connect("changed::" + Settings.COMMUTE_PROVIDER, function() {
        disable();
        enable();
    }));

    settings_signals.push(Settings.gsettings.connect("changed::" + Settings.COMMUTE_AUTOMATIC_SWITCH, function() {
        disable();
        enable();
    }));

    settings_signals.push(Settings.gsettings.connect("changed::" + Settings.COMMUTE_SWITCH_DISPLAYFORECAST, function() {
        disable();
        enable();
    }));

    _indicator = new RouteIndicator;
    Main.panel.addToStatusArea("RouteIndicator", _indicator);
}

function disable() {
    _indicator.destroy();
    // disconnect all signal listeners
    for(let i=0 ; i<settings_signals.length ; ++i) {
        Settings.gsettings.disconnect(settings_signals[i]);
    }
    settings_signals.length = 0;
}

