/**
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

const Lang = imports.lang;
const Soup = imports.gi.Soup;
const Me = imports.misc.extensionUtils.getCurrentExtension();
const Settings = Me.imports.settings;
const Clutter = imports.gi.Clutter;
const PopupMenu = imports.ui.popupMenu;


function getTimeFromSeconds(secondsValue) {
    // given the time as string in seconds returns the time in format hh:mm or mm minutes (if less than one hour)
    let value = parseInt(secondsValue);
    let hours = Math.floor(value / 3600);
    let minutes = Math.floor((value % 3600) / 60);
    if (hours > 0)
        return hours.toString() + _(":") + minutes.toString();
    else
        return minutes.toString() + _(" minutes");
}

const GoogleRouteProvider = new Lang.Class ({
    Name: 'GoogleRouteProvider',

    _init: function(actual_indicator) {
        this.indicator = actual_indicator
        this.routesData = []
        this.updateForecastData();
    },

    _onHTTPData: function (httpSession, message) {
        if (message.status_code != 200) return;
        this.routesData = JSON.parse(message.response_body.data);
        this.indicator.reloadMenus();
    },

    _getURL: function() {
        let URL = 'https://maps.googleapis.com/maps/api/directions/json?';
        return URL;
    },

    _getURLParams: function() {
        let params = {
            origin: this.indicator.flipOriginAndDestination ? Settings.gsettings.get_string(Settings.COMMUTE_DESTINATION) : Settings.gsettings.get_string(Settings.COMMUTE_ORIGIN),
            destination: this.indicator.flipOriginAndDestination ? Settings.gsettings.get_string(Settings.COMMUTE_ORIGIN) : Settings.gsettings.get_string(Settings.COMMUTE_DESTINATION),
            key: Settings.gsettings.get_string(Settings.COMMUTE_GOOGLE_KEY),
            alternatives: Settings.gsettings.get_boolean(Settings.COMMUTE_GOOGLE_ALTERNATIVES) ? 'true' : 'false',
            departure_time: 'now'
        }
        params[Settings.COMMUTE_GOOGLE_UNITS] = Settings.gsettings.get_string(Settings.COMMUTE_GOOGLE_UNITS);
        params[Settings.COMMUTE_GOOGLE_DRIVING_MODE] = Settings.gsettings.get_string(Settings.COMMUTE_GOOGLE_DRIVING_MODE);
        if (Settings.gsettings.get_string(Settings.COMMUTE_GOOGLE_AVOID) !== 'none') {
            params[Settings.COMMUTE_GOOGLE_AVOID] = Settings.gsettings.get_string(Settings.COMMUTE_GOOGLE_AVOID);
        }
        if (Settings.gsettings.get_string(Settings.COMMUTE_GOOGLE_TRANSIT_MODE) !== 'none') {
            params[Settings.COMMUTE_GOOGLE_TRANSIT_MODE] = Settings.gsettings.get_string(Settings.COMMUTE_GOOGLE_TRANSIT_MODE);
        }
        params[Settings.COMMUTE_GOOGLE_TRAFFIC_MODEL] = Settings.gsettings.get_string(Settings.COMMUTE_GOOGLE_TRAFFIC_MODEL);
        //global.log(_("Commute:") + JSON.stringify(params));
        return params;
    },

    getBrowserURL: function() {
        let URL = 'https://www.google.de/maps/dir/';
        let params = this._getURLParams();

        URL = URL + params.origin + _("/");
        URL = URL + params.destination;
        if ("driving" === Settings.gsettings.get_string(Settings.COMMUTE_GOOGLE_DRIVING_MODE)) {
            URL = URL + "/data=!4m2!4m1!3e0";
        } else if ("bicycling" === Settings.gsettings.get_string(Settings.COMMUTE_GOOGLE_DRIVING_MODE)) {
            URL = URL + "/data=!4m2!4m1!3e1";
        } else if ("walking" === Settings.gsettings.get_string(Settings.COMMUTE_GOOGLE_DRIVING_MODE)) {
            URL = URL + "/data=!4m2!4m1!3e2";
        } else if ("transit" === Settings.gsettings.get_string(Settings.COMMUTE_GOOGLE_DRIVING_MODE)) {
            URL = URL + "/data=!4m2!4m1!3e3";
        }

        //global.log(_("Commute: Used URL: ") + URL);
        return URL;
    },

    createRoutesMenus: function() {
        if ( this.routesData.length !== 0 ) {
            for(let r=0; r < this.routesData.routes.length; ++r) {
                let _itemRoute = new PopupMenu.PopupSubMenuMenuItem(_('via ') + this.routesData.routes[r].summary);
                this.indicator.menu.addMenuItem(_itemRoute);
                if (r===0) {
                    if (this.routesData.routes[r].legs[0].hasOwnProperty("duration_in_traffic")) {
                        this.indicator.routeDurationInfo.text = _("via ")
                            + this.routesData.routes[r].summary + _(" ")
                            + this.routesData.routes[r].legs[0].duration_in_traffic.text
                    } else {
                        this.indicator.routeDurationInfo.text = _("via ")
                            + this.routesData.routes[r].summary + _(" ")
                            + this.routesData.routes[r].legs[0].duration.text
                    }
                }

                global.log(_("Commute: via ") + this.routesData.routes[r].summary);
                for(let l=0; l < this.routesData.routes[r].legs.length; ++l) {
                    let duration = "";
                    if (this.routesData.routes[r].legs[l].hasOwnProperty("duration_in_traffic")) {
                        duration = _("  duration:") + this.routesData.routes[r].legs[l].duration_in_traffic.text;
                    } else {
                        duration = _("  duration:") + this.routesData.routes[r].legs[l].duration.text;
                    }
                    let distance = _("  distance:") + this.routesData.routes[r].legs[l].distance.text;
                    let itemDuration = new PopupMenu.PopupMenuItem(duration);
                    itemDuration.reactive = false;
                    let itemDistance = new PopupMenu.PopupMenuItem(distance);
                    itemDistance.reactive = false;
                    _itemRoute.menu.addMenuItem(itemDuration);
                    _itemRoute.menu.addMenuItem(itemDistance);
                }
            }
            this._separatorItem2 = new PopupMenu.PopupSeparatorMenuItem();
            this.indicator.menu.addMenuItem(this._separatorItem2);
        }
     },

    // called from external classes to trigger data update
    reloadData: function() {
        global.log("Commute: Trigger data refresh!");

        let _httpSession = new Soup.Session();
        let message = Soup.form_request_new_from_hash('GET', this._getURL(), this._getURLParams());
        //global.log(_("Commute: URL used: ") + message.get_uri().to_string(false));
        _httpSession.queue_message(message, Lang.bind(this, this._onHTTPData));
    },

    getForecastValues: function() {
        return this._forecastValues;
    },

    getMaxValue: function() {
        return this._maxValue;
    },

    getMinValue: function() {
        return this._minValue;
    },

    _onForecastData: function(index, httpSession, message) {
        this._responses = this._responses + 1;
        if (message.status_code != 200) return;
        let duration = 0;
        let routeForecastData = JSON.parse(message.response_body.data);
        if (routeForecastData.routes[0].legs[0].hasOwnProperty("duration_in_traffic")) {
            duration = Math.round(routeForecastData.routes[0].legs[0].duration_in_traffic.value / 60);
        } else {
            duration = Math.round(routeForecastData.routes[0].legs[0].duration.value / 60);
        }
        //global.log(_("Commute: URL used to get forecast data: ") + message.get_uri().to_string(false));
        global.log(_("Commute: on forecast data duration: ") + duration + _(" with index:") + index.toString());
        if (this._maxValue < duration) this._maxValue = duration;
        if (this._minValue > duration) this._minValue = duration;
        this._forecastValues[index] = duration;
        if (25 == this._responses) {
            // need to invalidate the extension to repaint the canvas
            global.log(_("Commute: on forecast data COMPLETE, reload menus!"));
            this.indicator.reloadMenus();
            this._responses = 0;
        }
    },

    updateForecastData: function() {
        if (0 < this._responses) return; // ongoing update
        this.httpSessions = []
        this._forecastValues = []
        this._maxValue = 0;
        this._minValue = 99999999;
        this._responses = 0;

        if (Settings.gsettings.get_boolean(Settings.COMMUTE_SWITCH_DISPLAYFORECAST)) {
            this._responses = this._responses + 1;
            let d = new Date();
            let params = this._getURLParams();
            for(let i=0; i < 24; ++i) {
                if (0 === i) {
                    params["departure_time"] = "now"
                } else {
                    params["departure_time"] = (Math.round(Date.now() / 1000 + i * 3600)).toString();
                }
                this.httpSessions[i] = new Soup.Session();
                let message = Soup.form_request_new_from_hash('GET', this._getURL(), params);
                this.httpSessions[i].queue_message(message, this._onForecastData.bind(this, (i + d.getHours()) % 24));
            }
        }
    }
});


const BingRouteProvider = new Lang.Class ({
    Name: 'BingRouteProvider',

    _init: function(actual_indicator) {
        this.indicator = actual_indicator
        this.updateForecastData();
        this.routesData = []
        this.geodestination = ''
        this.geoorigin = ''
        // we trigger here the lookup of origin and destination
        let URL = "http://dev.virtualearth.net/REST/v1/Locations"
        let paramsOrigin = {
            q: Settings.gsettings.get_string(Settings.COMMUTE_ORIGIN),
            key: Settings.gsettings.get_string(Settings.COMMUTE_BING_KEY)
        }
        let _httpSessionOrigin = new Soup.Session();
        let messageOrigin = Soup.form_request_new_from_hash('GET', URL, paramsOrigin);
        _httpSessionOrigin.queue_message(messageOrigin, Lang.bind(this, this._onHTTPDataOrigin));

        let paramsDestination = {
            q: Settings.gsettings.get_string(Settings.COMMUTE_DESTINATION),
            key: Settings.gsettings.get_string(Settings.COMMUTE_BING_KEY)
        }
        let _httpSessionDestination = new Soup.Session();
        let messageDestination = Soup.form_request_new_from_hash('GET', URL, paramsDestination);
        _httpSessionDestination.queue_message(messageDestination, Lang.bind(this, this._onHTTPDataDestination));
    },

    _onHTTPDataOrigin: function(httpSession, message) {
        if (message.status_code != 200) return;
        let originJSON = JSON.parse(message.response_body.data);
        this.geoorigin = _("pos.") + originJSON.resourceSets[0].resources[0].point.coordinates[0] +
            _("_") + originJSON.resourceSets[0].resources[0].point.coordinates[1];
    },

    _onHTTPDataDestination: function(httpSession, message) {
        if (message.status_code != 200) return;
        let destinationJSON = JSON.parse(message.response_body.data);
        this.geodestination = _("pos.") + destinationJSON.resourceSets[0].resources[0].point.coordinates[0] +
            _("_") + destinationJSON.resourceSets[0].resources[0].point.coordinates[1];
    },

    _onHTTPData: function (httpSession, message) {
        if (message.status_code != 200) return;
        this.routesData = JSON.parse(message.response_body.data);
        this.indicator.reloadMenus();
    },

    _getURL: function() {
        if ("Driving" === Settings.gsettings.get_string(Settings.COMMUTE_BING_TRAVELMODE)) {
            return 'http://dev.virtualearth.net/REST/V1/Routes/Driving?';
        } else if ("Walking" === Settings.gsettings.get_string(Settings.COMMUTE_BING_TRAVELMODE)) {
            return 'http://dev.virtualearth.net/REST/V1/Routes/Walking?';
        }
        return "";
    },

    _getURLParams: function() {
        let params = {
            o: 'json',
            'wp.1': this.indicator.flipOriginAndDestination ? Settings.gsettings.get_string(Settings.COMMUTE_DESTINATION) : Settings.gsettings.get_string(Settings.COMMUTE_ORIGIN),
            'wp.2': this.indicator.flipOriginAndDestination ? Settings.gsettings.get_string(Settings.COMMUTE_ORIGIN) : Settings.gsettings.get_string(Settings.COMMUTE_DESTINATION),
            ra: ("Driving" === Settings.gsettings.get_string(Settings.COMMUTE_BING_TRAVELMODE)) ? 'excludeItinerary' : 'all',
            maxSolutions: Settings.gsettings.get_boolean(Settings.COMMUTE_BING_ALTERNATIVES) ? '3' : '1',
            key: Settings.gsettings.get_string(Settings.COMMUTE_BING_KEY),
            avoid: Settings.gsettings.get_string(Settings.COMMUTE_BING_AVOID),
            distanceUnit: Settings.gsettings.get_string(Settings.COMMUTE_BING_DISTANCE_UNIT),
            travelMode: Settings.gsettings.get_string(Settings.COMMUTE_BING_TRAVELMODE)
        }
        if ("Driving" === Settings.gsettings.get_string(Settings.COMMUTE_BING_TRAVELMODE)) {
            params["optimize"] = Settings.gsettings.get_string(Settings.COMMUTE_BING_OPTIMIZE);
        } else {
            if ("timeWithTraffic" === Settings.gsettings.get_string(Settings.COMMUTE_BING_OPTIMIZE)) {
                params["optimize"] = "time";
            }
        }
        //global.log(_("Commute:") + JSON.stringify(params));
        return params;
    },

    getBrowserURL: function() {
        let URL = 'http://www.bing.com/mapspreview?rtp=';
        let params = this._getURLParams();

        URL = URL + this.geoorigin + _("__") + params['wp.1'] + _("__e_");
        URL = URL + _("~") + this.geodestination + _("__") + params['wp.2'] + _("__e_");
        if ("Walking" === Settings.gsettings.get_string(Settings.COMMUTE_BING_TRAVELMODE)) {
            URL = URL + "&mode=w";
        }
        return URL;
    },

    createRoutesMenus: function() {
        if ( this.routesData.length !== 0 ) {
            for(let r=0; r < this.routesData.resourceSets[0].resources.length; ++r) {
                let _itemRoute = new PopupMenu.PopupSubMenuMenuItem(_('via ') + this.routesData.resourceSets[0].resources[r].routeLegs[0].description);
                _itemRoute.label.style_class = this.routesData.resourceSets[0].resources[r].trafficCongestion;
                this.indicator.menu.addMenuItem(_itemRoute);
                if (r===0) {
                    this.indicator.routeDurationInfo.text = _("via ") + this.routesData.resourceSets[0].resources[r].routeLegs[0].description
                     + _(" ") + getTimeFromSeconds(this.routesData.resourceSets[0].resources[r].travelDurationTraffic);
                     this.indicator.routeDurationInfo.style_class = this.routesData.resourceSets[0].resources[r].trafficCongestion;
                }

                global.log(_("Commute: via ") + this.routesData.resourceSets[0].resources[r].routeLegs[0].description);
                let duration = _("  duration:") + getTimeFromSeconds(this.routesData.resourceSets[0].resources[r].travelDurationTraffic);
                let distance = _("  distance:") + this.routesData.resourceSets[0].resources[r].travelDistance + _(" km");
                let itemDuration = new PopupMenu.PopupMenuItem(duration);
                itemDuration.reactive = false;
                let itemDistance = new PopupMenu.PopupMenuItem(distance);
                itemDistance.reactive = false;
                _itemRoute.menu.addMenuItem(itemDuration);
                _itemRoute.menu.addMenuItem(itemDistance);
            }
            this._separatorItem2 = new PopupMenu.PopupSeparatorMenuItem();
            this.indicator.menu.addMenuItem(this._separatorItem2);
        }
     },

    // called from external classes to trigger data update
    reloadData: function() {
        global.log("Commute: Trigger data refresh!");

        let _httpSession = new Soup.Session();
        let message = Soup.form_request_new_from_hash('GET', this._getURL(), this._getURLParams());
        //global.log(_("Commute: URL used: ") + message.get_uri().to_string(false));
        _httpSession.queue_message(message, Lang.bind(this, this._onHTTPData));
    },

    getForecastValues: function() {
        return this._forecastValues;
    },

    getMaxValue: function() {
        return this._maxValue;
    },

    getMinValue: function() {
        return this._minValue;
    },

    _onForecastData: function(index, httpSession, message) {
        this._responses = this._responses + 1;
        if (message.status_code != 200) return;
        let duration = 0;
        let routeForecastData = JSON.parse(message.response_body.data);
        //global.log(_("Commute: URL used to get forecast data: ") + message.get_uri().to_string(false));
        duration = Math.round(routeForecastData.resourceSets[0].resources[0].travelDurationTraffic / 60);
        global.log(_("Commute: on forecast data duration: ") + duration + _(" with index:") + index.toString());
        if (this._maxValue < duration) this._maxValue = duration;
        if (this._minValue > duration) this._minValue = duration;
        this._forecastValues[index] = duration;
        if (25 == this._responses) {
            // need to invalidate the extension to repaint the canvas
            global.log(_("Commute: on forecast data COMPLETE, reload menus!"));
            this.indicator.reloadMenus();
            this._responses = 0;
        }
    },

    updateForecastData: function() {
        if (0 < this._responses) return; // ongoing update
        this.httpSessions = []
        this._forecastValues = []
        this._maxValue = 0;
        this._minValue = 99999999;
        this._responses = 0;

        if (Settings.gsettings.get_boolean(Settings.COMMUTE_SWITCH_DISPLAYFORECAST)) {
            this._responses = this._responses + 1;
            let initialDateTime = new Date(Date.now());
            global.log(_("Commute: initial date time for forecast data:") + initialDateTime.toISOString());
            let params = this._getURLParams();
            for(let i=0; i < 24; ++i) {
                let d = new Date(initialDateTime.getTime() + i * 3600000);
                params["dt"] = d.toISOString();
                this.httpSessions[i] = new Soup.Session();
                let message = Soup.form_request_new_from_hash('GET', this._getURL(), params);
                this.httpSessions[i].queue_message(message, this._onForecastData.bind(this, d.getHours()));
            }
        }
    }
});

function init() {
}


